==> PyMSNt-0.1.tar.gz: PyMSNt/src/jabw.py <==
# Copyright 2004 James Bunton <james@delx.cjb.net>
# Licensed for distribution under the GPL version 2, check COPYING for details

from twisted.xish.domish import Element
from twisted.protocols.jabber.jid import JID
import debug


def sendMessage(pytrans, to, fro, body, mtype=None):
	""" Sends a Jabber message """
	debug.log("jabw: Sending a Jabber message \"%s\" \"%s\" \"%s\" \"%s\"" % (to, fro, body, mtype))
	el = Element((None, "message"))
	el.attributes["to"] = to
	el.attributes["from"] = fro
	if(mtype):
		el.attributes["type"] = mtype
	b = el.addElement("body")
	b.addContent(body)
	pytrans.send(el)



class JabberConnection:
	""" A class to handle a Jabber "Connection", ie, the Jabber side of the gateway.
	If you want to send a Jabber event, this is the place, and this is where incoming
	Jabber events for a session come to. """
	
	def __init__(self, pytrans, jabberID):
		debug.log("User: %s - Constructing JabberConnection" % (jabberID))
		self.pytrans = pytrans
		self.jabberID = jabberID
		self.pytrans.addObserver(self.jabberID, "/message", self.onMessage)
		self.pytrans.addObserver(self.jabberID, "/presence", self.onPresence)
		debug.log("User: %s - JabberConnection constructed" % (jabberID))
	
	def removeMe(self):
		""" Cleanly deletes the object """
		debug.log("User: %s - Removing JabberConnection" % (self.jabberID))
		self.pytrans.remObserver(self.onMessage)
		self.pytrans.remObserver(self.onPresence)
		self.pytrans = None
		debug.log("User: %s - JabberConnection removed" % (self.jabberID))
	
	def checkFrom(self, el):
		""" Checks to see that this packet was intended for this object """
		fro = el.getAttribute("from")
		froj = JID(fro)
		
		return (froj.userhost() == self.jabberID) # Compare with the Jabber ID that we're looking at
	
	def sendMessage(self, to, fro, body, mtype=None):
		""" Sends a Jabber message """
		debug.log("User: %s - JabberConnection sending message \"%s\" \"%s\" \"%s\" \"%s\"" % (self.jabberID, to, fro, body, mtype))
		sendMessage(self.pytrans, to, fro, body, mtype)
	
	def sendPresence(self, to, fro, show=None, status=None, priority=None, ptype=None):
		""" Sends a Jabber presence packet """
		debug.log("User: %s - JabberConnection sending presence \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\"" % (self.jabberID, to, fro, show, status, priority, ptype))
		el = Element((None, "presence"))
		el.attributes["to"] = to
		el.attributes["from"] = fro
		if(ptype):
			el.attributes["type"] = ptype
		if(show):
			s = el.addElement("show")
			s.addContent(show)
		if(status):
			s = el.addElement("status")
			s.addContent(status)
		if(priority):
			s = el.addElement("priority")
			s.addContent(priority)
		self.pytrans.send(el)
	
	def onMessage(self, el):
		""" Handles incoming message packets """
		if(not self.checkFrom(el)): return
		debug.log("User: %s - JabberConnection received message packet" % (self.jabberID))
		fro = el.getAttribute("from")
		froj = JID(fro)
		to = el.getAttribute("to")
		
		mtype = el.getAttribute("type")
		body = None
		for child in el.elements():
			if(child.name == "body"):
				body = str(child)
		
		debug.log("User: %s - JabberConnection parsed message packet \"%s\" \"%s\" \"%s\" \"%s\" \"%s\"" % (self.jabberID, froj.userhost(), to, froj.resource, mtype, body))
		self.messageReceived(froj.userhost(), to, froj.resource, mtype, body)
	
	def onPresence(self, el):
		""" Handles incoming presence packets """
		if(not self.checkFrom(el)): return
		debug.log("User: %s - JabberConnection received presence packet" % (self.jabberID))
		fro = el.getAttribute("from")
		froj = JID(fro)
		to = el.getAttribute("to")
		toj = JID(to)
		
		# Grab the contents of the <presence/> packet
		ptype = el.getAttribute("type")
		if(ptype in ["subscribe", "subscribed", "unsubscribe", "unsubscribed"]):
			debug.log("User: %s - JabberConnection parsed subscription presence packet \"%s\" \"%s\"" % (self.jabberID, toj.userhost(), ptype))
			self.subscriptionReceived(toj.userhost(), ptype)
		else:
			if(to.find('@') > 0): return # Ignore presence packets sent to users
			status = None
			show = None
			priority = None
			for child in el.elements():
				if(child.name == "status"):
					status = str(child)
				elif(child.name == "show"):
					show = str(child)
				elif(child.name == "priority"):
					priority = str(child)
			
			debug.log("User: %s - JabberConnection parsed presence packet \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\"" % (self.jabberID, froj.userhost(), froj.resource, priority, ptype, show, status))
			self.presenceReceived(froj.userhost(), froj.resource, priority, ptype, show, status)
	
	
	
	def messageReceived(self, source, dest, resource, mtype, body):
		""" Override this method to be notified when a message is received """
		pass
	
	def presenceReceived(self, source, resource, priority, ptype, show, status):
		""" Override this method to be notified when presence is received """
		pass
	
	def subscriptionReceived(self, source, subtype):
		""" Override this method to be notified when a subscription packet is received """
		pass

==> PyMSNt-0.2.tar.gz: PyMSNt/src/jabw.py <==
# Copyright 2004 James Bunton <james@delx.cjb.net>
# Licensed for distribution under the GPL version 2, check COPYING for details

from twisted.xish.domish import Element
from twisted.protocols.jabber.jid import JID
import debug


def sendMessage(pytrans, to, fro, body, mtype=None):
	""" Sends a Jabber message """
	debug.log("jabw: Sending a Jabber message \"%s\" \"%s\" \"%s\" \"%s\"" % (to, fro, body, mtype))
	el = Element((None, "message"))
	el.attributes["to"] = to
	el.attributes["from"] = fro
	if(mtype):
		el.attributes["type"] = mtype
	b = el.addElement("body")
	b.addContent(body)
	pytrans.send(el)


def u(text):
	# FIXME. I'm sure Jabber should support UTF-8. Must be a problem with Twisted xmlstream
	if(not (text.__class__ in [str, unicode])): return None
	newtext = ""
	for c in text:
		i = ord(c)
		if(i >= 128):
			i = ord(' ')
		newtext += chr(i)
	return newtext



class JabberConnection:
	""" A class to handle a Jabber "Connection", ie, the Jabber side of the gateway.
	If you want to send a Jabber event, this is the place, and this is where incoming
	Jabber events for a session come to. """
	
	def __init__(self, pytrans, jabberID):
		debug.log("User: %s - Constructing JabberConnection" % (jabberID))
		self.pytrans = pytrans
		self.jabberID = jabberID
		self.pytrans.addObserver(self.jabberID, "/message", self.onMessage)
		self.pytrans.addObserver(self.jabberID, "/presence", self.onPresence)
		debug.log("User: %s - JabberConnection constructed" % (jabberID))
	
	def removeMe(self):
		""" Cleanly deletes the object """
		debug.log("User: %s - Removing JabberConnection" % (self.jabberID))
		self.pytrans.remObserver(self.onMessage)
		self.pytrans.remObserver(self.onPresence)
		self.pytrans = None
		debug.log("User: %s - JabberConnection removed" % (self.jabberID))
	
	def checkFrom(self, el):
		""" Checks to see that this packet was intended for this object """
		fro = el.getAttribute("from")
		froj = JID(fro)
		
		return (froj.userhost() == self.jabberID) # Compare with the Jabber ID that we're looking at
	
	def sendMessage(self, to, fro, body, mtype=None):
		""" Sends a Jabber message """
		body = u(body)
		debug.log("User: %s - JabberConnection sending message \"%s\" \"%s\" \"%s\" \"%s\"" % (self.jabberID, to, fro, body, mtype))
		sendMessage(self.pytrans, to, fro, body, mtype)
	
	def sendPresence(self, to, fro, show=None, status=None, priority=None, ptype=None):
		""" Sends a Jabber presence packet """
		status = u(status)
		debug.log("User: %s - JabberConnection sending presence \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\"" % (self.jabberID, to, fro, show, status, priority, ptype))
		el = Element((None, "presence"))
		el.attributes["to"] = to
		el.attributes["from"] = fro
		if(ptype):
			el.attributes["type"] = ptype
		if(show):
			s = el.addElement("show")
			s.addContent(show)
		if(status):
			s = el.addElement("status")
			s.addContent(status)
		if(priority):
			s = el.addElement("priority")
			s.addContent(priority)
		self.pytrans.send(el)
	
	def onMessage(self, el):
		""" Handles incoming message packets """
		if(not self.checkFrom(el)): return
		debug.log("User: %s - JabberConnection received message packet" % (self.jabberID))
		fro = el.getAttribute("from")
		froj = JID(fro)
		to = el.getAttribute("to")
		toj = JID(to)
		
		mtype = el.getAttribute("type")
		body = None
		invite = None
		for child in el.elements():
			if(child.name == "body"):
				body = str(child)
			if(child.name == "x"):
				if(child.uri == "jabber:x:conference"):
					invite = child.getAttribute("jid") # The room the contact is being invited to
		
		if(invite):
			debug.log("User: %s - JabberConnection parsed message groupchat invite packet \"%s\" \"%s\" \"%s\" \"%s\"" % (self.jabberID, froj.userhost(), to, froj.resource, invite))
			self.inviteReceived(froj.userhost(), froj.resource, toj.userhost(), toj.resource, invite)
		else:
			debug.log("User: %s - JabberConnection parsed message packet \"%s\" \"%s\" \"%s\" \"%s\" \"%s\"" % (self.jabberID, froj.userhost(), to, froj.resource, mtype, body))
			self.messageReceived(froj.userhost(), froj.resource, toj.userhost(), toj.resource, mtype, body)
	
	def onPresence(self, el):
		""" Handles incoming presence packets """
		if(not self.checkFrom(el)): return
		debug.log("User: %s - JabberConnection received presence packet" % (self.jabberID))
		fro = el.getAttribute("from")
		froj = JID(fro)
		to = el.getAttribute("to")
		toj = JID(to)
		
		# Grab the contents of the <presence/> packet
		ptype = el.getAttribute("type")
		if(ptype in ["subscribe", "subscribed", "unsubscribe", "unsubscribed"]):
			debug.log("User: %s - JabberConnection parsed subscription presence packet \"%s\" \"%s\"" % (self.jabberID, toj.userhost(), ptype))
			self.subscriptionReceived(toj.userhost(), ptype)
		else:
			status = None
			show = None
			priority = None
			for child in el.elements():
				if(child.name == "status"):
					status = str(child)
				elif(child.name == "show"):
					show = str(child)
				elif(child.name == "priority"):
					priority = str(child)
			
			debug.log("User: %s - JabberConnection parsed presence packet \"%s\" \"%s\" \"%s\" \"%s\" \"%s\" \"%s\"" % (self.jabberID, froj.userhost(), froj.resource, priority, ptype, show, status))
			self.presenceReceived(froj.userhost(), froj.resource, toj.userhost(), toj.resource, priority, ptype, show, status)
	
	
	
	def messageReceived(self, source, resource, dest, destr, mtype, body):
		""" Override this method to be notified when a message is received """
		pass
	
	def inviteReceived(self, source, resource, dest, destr, roomjid):
		""" Override this method to be notified when an invitation is received """
		pass
	
	def presenceReceived(self, source, resource, to, tor, priority, ptype, show, status):
		""" Override this method to be notified when presence is received """
		pass
	
	def subscriptionReceived(self, source, subtype):
		""" Override this method to be notified when a subscription packet is received """
		pass
